------------------------------------------------------------
-- Initialize.lua
--
-- Abin
-- 2012/1/03
------------------------------------------------------------

local _G = _G
local type = type
local format = format
local SlashCmdList = SlashCmdList
local ipairs = ipairs

local addonName, addon = ...
local L = addon.L

local initDone

function addon:IsInitialized()
	return initDone
end

local function InitializeAddon()
	if initDone then
		return
	end

	-- Initialize data tables
	if type(CompactRaidDB) == "table" then
		addon.db = CompactRaidDB
	else
		CompactRaidDB = addon.db
	end

	if type(CompactRaidCharDB) == "table" then
		addon.chardb = CompactRaidCharDB
	else
		CompactRaidCharDB = addon.chardb
	end

	if type(addon.db.modules) ~= "table" then
		addon.db.modules = {}
	end

	if type(addon.chardb.modules) ~= "table" then
		addon.chardb.modules = {}
	end

	-- Remove obsoleted data
	local obsoleted
	for _, obsoleted in ipairs(addon.OBSOLETE_MODULES) do
		addon.db.modules[obsoleted] = nil
		addon.chardb.modules[obsoleted] = nil
	end

	-- Initialize addon data
	addon.db.scale = addon:NormalizeNumber(addon.db.scale, 25, 300, 100)
	addon.db.width = addon:NormalizeNumber(addon.db.width, 24, 120, 64)
	addon.db.height = addon:NormalizeNumber(addon.db.height, 24, 120, 36)
	addon.db.spacing = addon:NormalizeNumber(addon.db.spacing, 0, 10, 1)
	addon.db.powerBarHeight = addon:NormalizeNumber(addon.db.powerBarHeight, 0, 10, 2)
	addon.db.showtooltip = addon:NormalizeNumber(addon.db.showtooltip, 0, 2, 1)
	addon.db.nameWidthLimit = addon:NormalizeNumber(addon.db.nameWidthLimit, 0, 100, 75)
	addon.db.nameHeight = addon:NormalizeNumber(addon.db.nameHeight, 2, 20, 12)
	addon.db.containerAlpha = addon:NormalizeNumber(addon.db.containerAlpha, 0, 100, 75)
	addon.db.containerBorderSize = addon:NormalizeNumber(addon.db.containerBorderSize, 0, 24, 12)
	addon.db.classfilter = addon:NormalizeNumber(addon.db.classfilter, 0, 1, 0)
	if type(addon.db.classorder) ~= "string" then
		addon.db.classorder = addon:GetDefaultClassOrder()
	end

	addon.db.namer = addon:NormalizeNumber(addon.db.namer, 0, 1, 1)
	addon.db.nameg = addon:NormalizeNumber(addon.db.nameg, 0, 1, 1)
	addon.db.nameb = addon:NormalizeNumber(addon.db.nameb, 0, 1, 1)
	addon.db.healthr = addon:NormalizeNumber(addon.db.healthr, 0, 1, 0.5)
	addon.db.healthg = addon:NormalizeNumber(addon.db.healthg, 0, 1, 0.5)
	addon.db.healthb = addon:NormalizeNumber(addon.db.healthb, 0, 1, 1)
	addon.db.powerr = addon:NormalizeNumber(addon.db.powerr, 0, 1, 0)
	addon.db.powerg = addon:NormalizeNumber(addon.db.powerg, 0, 1, 0)
	addon.db.powerb = addon:NormalizeNumber(addon.db.powerb, 0, 1, 1)

	-- Initialize addon
	addon:UpdateScale()
	addon:UpdateUnitFrameSize()
	addon:UpdateNameWidth()
	addon:UpdateNameHeight()
	addon:UpdateStatusBarBkgnd()
	addon:UpdateNameColorMode()
	addon:UpdateNameOffsets()
	addon:UpdateHealthColorMode()
	addon:UpdatePowerBarHeight()
	addon:UpdatePowerColorMode()
	addon:UpdateSoloMode()
	addon:UpdatePartyMode()
	addon:UpdatePartyPetMode()
	addon:UpdateRaidGroupFilter()
	addon:UpdateRaidGroupClassOrder()
	addon:UpdateToolFrameVisibility()
	addon:UpdateAuraMode()
	addon:UpdateVisibility()
	addon:UpdateSpacing()
	addon:UpdateContainerAlpha()
	addon:UpdateContainerBorderSize()
	addon:UpdateClassOrderFrame()

	-- At this point the addon itself has been initialized, we can start processing modules now
	initDone = 1

	-- Register frames to Clique if presents
	addon:CliqueRegister()

	------------------------------------------------------------
	-- Register slash command to toggle the option frame
	------------------------------------------------------------
	SLASH_COMPACTRAID1 = "/compactraid"
	SLASH_COMPACTRAID2 = "/craid"
	SlashCmdList["COMPACTRAID"] = function() addon.optionFrame:Toggle() end

	addon:Print(format(L["load prompt"], addon.version))
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:SetScript("OnEvent", function(self, event, arg1)
	if event == "ADDON_LOADED" then
		if arg1 == addonName then
			InitializeAddon()
			self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
		end

		if addon:IsInitialized() then
			addon:ProcessModuleLoading(arg1)
		end

	elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		addon:NotifyTalentSwitch()
	end
end)