------------------------------------------------------------
-- OptionFrame.lua
--
-- Abin
-- 2012/1/03
------------------------------------------------------------

local _G = _G
local type = type
local strsplit = strsplit
local CLASS_BUTTONS = CLASS_BUTTONS

local _, addon = ...
local L = addon.L
local templates = addon.optionTemplates

-------------------------------------------------------
-- The option frame
-------------------------------------------------------

local frame = UICreateInterfaceOptionPage("CompactRaidOptionFrame", addon.name, nil, nil, UIParent)
addon.optionFrame = frame
frame:SetDialogStyle("TITLE_DIALOG", true)
frame:SetSize(858, 660)
frame:SetPoint("CENTER")

function frame:AddCategory(title, optionPage, parentTitle)
	if type(title) ~= "string" or type(optionPage) ~= "table" then
		return
	end

	optionPage:Hide()
	optionPage:SetParent(self.pageContainer)
	optionPage:ClearAllPoints()
	optionPage:SetAllPoints(self.pageContainer)

	local level = 1
	local position
	if type(parentTitle) == "string" then
		position = self.categories:FindData(parentTitle, function(data, text) return data.title == text end)
		local data = self.categories:GetData(position)
		if data then
			level = data.level + 1
		end
	end
	self.categories:InsertData({ title = title, page = optionPage, level = level }, position and position + 1)
end

-------------------------------------------------------
-- Left panel: container of module categories
-------------------------------------------------------

local leftPanel = CreateFrame("Frame", frame:GetName().."LeftPanel", frame)
frame.leftPanel = leftPanel
leftPanel:SetBackdrop({ tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }})
leftPanel:SetBackdropBorderColor(0.75, 0.75, 0.75, 0.75)
leftPanel:SetPoint("TOPLEFT", 24, -32)
leftPanel:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", 220, 24)

local list = UICreateVirtualScrollList(leftPanel:GetName().."List", leftPanel, 10, 1)
frame.categories = list
list:SetPoint("TOPLEFT", 4, -5)
list:SetPoint("BOTTOMRIGHT", -4, 4)

function list:OnButtonCreated(button)
	local text = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	button.text = text
end

function list:OnButtonUpdate(button, data)
	local text = button.text
	text:ClearAllPoints()
	text:SetPoint("LEFT", (data.level - 1) * 8 + 5, 0)
	button.text:SetText(data.title)
	if self.checkedTexture.button == button then
		self.curButton = button
		button.text:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	end
end

function list:OnSelectionChanged(position, data)
	if self.curPage then
		self.curPage:Hide()
	end

	local page = data.page
	self.curPage = page
	if page then
		page:Show()
	end

	if self.curButton then
		self.curButton.text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
	end

	self.curButton = self.checkedTexture.button
	if self.curButton then
		self.curButton.text:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	end
end

function list:OnButtonEnter(button, data, motion)
	button.text:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
end

function list:OnButtonLeave(button, data, motion)
	if button ~= self.curButton then
		button.text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
	end
end

-------------------------------------------------------
-- Right panel: container of option pages
-------------------------------------------------------

local rightPanel = CreateFrame("Frame", frame:GetName().."RightPanel", frame)
frame.rightPanel = rightPanel
rightPanel:SetBackdrop({ tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }})
rightPanel:SetBackdropBorderColor(0.75, 0.75, 0.75, 0.75)
rightPanel:SetPoint("TOPLEFT", leftPanel, "TOPRIGHT", 12, 0)
rightPanel:SetPoint("BOTTOMRIGHT", -24, 24)

local scrollFrame, pageContainer = templates:CreateScrollFrame(rightPanel:GetName().."ScrollFrame", rightPanel, nil, 1)
frame.pageContainer = pageContainer
scrollFrame:SetPoint("TOPLEFT", 6, -6)
scrollFrame:SetPoint("BOTTOMRIGHT", -27, 5)

-------------------------------------------------------
-- The first option page
-------------------------------------------------------

local page = templates:CreateOptionFrame(frame:GetName().."MainPage", addon.name, addon.version, L["desc"])
addon.optionPage = page
frame:AddCategory(L["core module"].."|TInterface\\OptionsFrame\\UI-OptionsFrame-NewFeatureIcon:0:0:0:-1|t", page)
list:SetSelection(1)

------------------------------------------------------------
-- General option items
------------------------------------------------------------

local group = page:CreateMultiSelectionGroup(L["general options"])
page:AnchorToTopLeft(group)
group:AddButton(L["lock position"], "lock")
group:AddButton(L["invert bar color"], "invertColor")
group:AddButton(L["hide bar background"], "hidebarbkgnd")
group:AddButton(L["hide solo"], "hidesolo", 1)
group:AddButton(L["hide party"], "hideparty", 1)
group:AddButton(L["hide party pets"], "hidepartypets", 1)

function group:OnCheckInit(value)
	return addon.db[value]
end

function group:OnCheckChanged(value, checked)
	addon.db[value] = checked

	if value == "invertColor" then
		addon:UpdateHealthColorMode()
		addon:UpdatePowerColorMode()
	elseif value == "hidebarbkgnd" then
		addon:UpdateStatusBarBkgnd()
	elseif value == "hidesolo" then
		addon:UpdateSoloMode()
		addon:UpdateVisibility()
	elseif value == "hideparty" then
		addon:UpdatePartyMode()
		addon:UpdateVisibility()
	elseif value == "hidepartypets" then
		addon:UpdatePartyPetMode()
	end
end

------------------------------------------------------------
-- Raid option items
------------------------------------------------------------
local anchor = group[-1]
group = page:CreateMultiSelectionGroup(L["raid options"])
group:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -16)
group:AddButton(L["align raid units horizontally"], "raidhoriz", 1)

function group:OnCheckInit(value)
	return addon.db.raidhoriz
end

function group:OnCheckChanged(value, checked)
	addon.db.raidhoriz = checked
	addon:UpdateRaidGroupOrientation()
end

anchor = group[-1]
group = page:CreateSingleSelectionGroup(nil, 1)
group:AddButton(L["group by subgroups"], 0, 1):ClearAllPoints()
group[1]:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT")
group:AddButton(L["group by classes"], 1, 1)

function group:OnCheckInit(value)
	return addon.db.classfilter == value
end

function group:OnSelectionChanged(value)
	addon.db.classfilter = value
	addon:UpdateRaidGroupFilter()
	addon:UpdateToolFrameVisibility()
	addon:UpdateClassOrderFrame()
end

------------------------------------------------------------
-- Class order frame
------------------------------------------------------------
page.prevAnchor = group[1]

local classOrderFrame = templates:CreateClassOrderFrame(page:GetName().."ClassOrderFrame", page, addon:GetDefaultClassOrder())
page.classOrderFrame = classOrderFrame
classOrderFrame:SetPoint("TOPLEFT", group[1], "BOTTOMLEFT")

function classOrderFrame:OnClassOrderInit()
	return addon.db.classorder
end

function classOrderFrame:OnClassOrderChanged(classOrder)
	addon.db.classorder = classOrder
	addon:UpdateRaidGroupClassOrder()
end

function addon:UpdateClassOrderFrame()
	local tooltipGroup = page.tooltipGroup
	page.tooltipGroup:ClearAllPoints()
	if addon.db.classfilter ~= 1 then
		tooltipGroup:SetPoint("TOPLEFT", page.prevAnchor, "BOTTOMLEFT", 0, -16)
		classOrderFrame:Hide()
	else
		classOrderFrame:Show()
		tooltipGroup:SetPoint("TOPLEFT", classOrderFrame, "BOTTOMLEFT", 0, -16)
	end
end

------------------------------------------------------------
-- Tooltip options
------------------------------------------------------------
group = page:CreateSingleSelectionGroup(L["Display tooltips"], 1)
page.tooltipGroup = group
group:SetPoint(classOrderFrame:GetPoint(1))
group:AddButton(ALWAYS, 2)
group:AddButton(L["out of combat"], 1)
group:AddButton(NEVER, 0)

function group:OnCheckInit(value)
	return addon.db.showtooltip == value
end

function group:OnSelectionChanged(value)
	addon.db.showtooltip = value
end

------------------------------------------------------------
-- Frame sizing options
------------------------------------------------------------

local sizingLabel = page:CreateFontString(nil, "ARTWORK", "GameFontNormal")
sizingLabel:SetPoint("TOPLEFT", group[1], "BOTTOMLEFT", 0, -16)
sizingLabel:SetText(L["frame sizing"])

local scaleSlider = page:CreateSmallSlider(L["frame scale"], 25, 300, 5, "%d%%", 1)
scaleSlider:SetPoint("TOPLEFT", sizingLabel, "BOTTOMLEFT", 8, -28)

function scaleSlider:OnSliderInit()
	return addon.db.scale
end

function scaleSlider:OnSliderChanged(value)
	addon.db.scale = value
	addon:UpdateScale()
end

local spacingSlider = page:CreateSmallSlider(L["unit spacing"], 0, 10, 1, nil, 1)
spacingSlider:SetPoint("LEFT", scaleSlider, "RIGHT", 20, 0)

function spacingSlider:OnSliderInit()
	return addon.db.spacing
end

function spacingSlider:OnSliderChanged(value)
	addon.db.spacing = value
	addon:UpdateSpacing()
end

local widthSlider = page:CreateSmallSlider(L["unit width"], 24, 120, 2, nil, 1)
widthSlider:SetPoint("TOPLEFT", scaleSlider, "BOTTOMLEFT", 0, -38)

function widthSlider:OnSliderInit()
	return addon.db.width
end

function widthSlider:OnSliderChanged(value)
	addon.db.width = value
	addon:UpdateUnitFrameSize()
end

local heightSlider = page:CreateSmallSlider(L["unit height"], 24, 120, 2, nil, 1)
heightSlider:SetPoint("LEFT", widthSlider, "RIGHT", 20, 0)

function heightSlider:OnSliderInit()
	return addon.db.height
end

function heightSlider:OnSliderChanged(value)
	addon.db.height = value
	addon:UpdateUnitFrameSize()
end

local powerBarSlider = page:CreateSmallSlider(L["mana height"], 0, 10)
powerBarSlider:SetPoint("TOPLEFT", widthSlider, "BOTTOMLEFT", 0, -38)

function powerBarSlider:OnSliderInit()
	return addon.db.powerBarHeight
end

function powerBarSlider:OnSliderChanged(value)
	addon.db.powerBarHeight = value
	addon:UpdatePowerBarHeight()
end

------------------------------------------------------------
-- Health bar color options
------------------------------------------------------------

-- just to simplify the code...
local function CreateColorSelectionGroup(text, dbKey, dbColorKey, func)
	local group = templates:CreateCustomColorGroup(page, text)
	group.dbKey, group.dbColorKey, group.func = dbKey, dbColorKey, func

	group.OnColorRequest = function(self)
		local key = self.dbColorKey
		return addon.db[key.."r"], addon.db[key.."g"], addon.db[key.."b"]
	end

	group.OnColorChange = function(self, r, g, b)
		local key = self.dbColorKey
		addon.db[key.."r"], addon.db[key.."g"], addon.db[key.."b"] = r, g, b
	end

	group.OnOptionInit = function(self, value)
		if addon.db[self.dbKey] then
			return value == 0
		else
			return value == 1
		end
	end

	group.OnOptionChange = function(self, value)
		if value == 0 then
			addon.db[self.dbKey] = 1
		else
			addon.db[self.dbKey] = nil
		end
	end

	group.OnApplyOption = function(self)
		addon[self.func](addon)
	end

	return group
end

anchor = group[1]
group = CreateColorSelectionGroup(L["health bar color"], "forceHealthColor", "health", "UpdateHealthColorMode")
group:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -192)

anchor = group[1]
group = CreateColorSelectionGroup(L["power bar color"], "forcePowerColor", "power", "UpdatePowerColorMode")
group:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -16)

------------------------------------------------------------
-- Name text options
------------------------------------------------------------

anchor = group[1]
group = CreateColorSelectionGroup(L["name text options"], "forceNameColor", "name", "UpdateNameColorMode")
group:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -16)

local function CreateNameTextOptionSlider(text, minVal, maxVal, step, valueFormat, dbKey, func)
	local slider = page:CreateSmallSlider(text, minVal, maxVal, step, valueFormat)
	slider.dbKey, slider.func = dbKey, func

	slider.OnSliderInit = function(self)
		return addon.db[self.dbKey] or 0
	end

	slider.OnSliderChanged = function(self, value)
		addon.db[self.dbKey] = value
		addon[self.func](addon)
	end
	return slider
end

local nameLenSlider = CreateNameTextOptionSlider(L["text length"], 0, 100, 5, "%d%%", "nameWidthLimit", "UpdateNameWidth")
nameLenSlider:SetPoint("TOPLEFT", group[1], "BOTTOMLEFT", 8, -26)

local nameHeightSlider = CreateNameTextOptionSlider(L["text height"], 2, 20, nil, nil, "nameHeight", "UpdateNameHeight")
nameHeightSlider:SetPoint("LEFT", nameLenSlider, "RIGHT", 20, 0)

local nameXOffSlider = CreateNameTextOptionSlider(L["x-offset"], -20, 20, nil, nil, "nameXOffset", "UpdateNameOffsets")
nameXOffSlider:SetPoint("TOPLEFT", nameLenSlider, "BOTTOMLEFT", 0, -34)

local nameYOffSlider = CreateNameTextOptionSlider(L["y-offset"], -20, 20, nil, nil, "nameYOffset", "UpdateNameOffsets")
nameYOffSlider:SetPoint("LEFT", nameXOffSlider, "RIGHT", 20, 0)

------------------------------------------------------------
-- Container options
------------------------------------------------------------

local containerLabel = page:CreateFontString(nil, "ARTWORK", "GameFontNormal")
containerLabel:SetPoint("TOPLEFT", group[1], "BOTTOMLEFT", 0, -110)
containerLabel:SetText(L["frame container options"])

local containerAlphaSlider = page:CreateSmallSlider(L["background alpha"], 0, 100, 5, "%d%%")
containerAlphaSlider:SetPoint("TOPLEFT", containerLabel, "BOTTOMLEFT", 8, -28)

function containerAlphaSlider:OnSliderInit()
	return addon.db.containerAlpha
end

function containerAlphaSlider:OnSliderChanged(value)
	addon.db.containerAlpha = value
	addon:UpdateContainerAlpha()
end

local borderSizeSlider = page:CreateSmallSlider(L["border size"], 0, 24, 1, nil, 1)
borderSizeSlider:SetPoint("LEFT", containerAlphaSlider, "RIGHT", 20, 0)

function borderSizeSlider:OnSliderInit()
	return addon.db.containerBorderSize
end

function borderSizeSlider:OnSliderChanged(value)
	addon.db.containerBorderSize = value
	addon:UpdateContainerBorderSize()
end

------------------------------------------------------------
-- Aura options
------------------------------------------------------------
group = page:CreateMultiSelectionGroup(L["aura options"])
group:SetPoint("TOPLEFT", containerLabel, "BOTTOMLEFT", 0, -66)

group:AddButton(L["hide buffs"], "hideBuffs")
group:AddButton(L["hide debuffs"], "hideDebuffs")
group:AddButton(L["only show dispellable debuffs"], "onlyDispellable")
group:AddButton(L["hide dispels"], "hideDispels")

function group:OnCheckInit(value)
	return addon.db[value]
end

function group:OnCheckChanged(value, checked)
	addon.db[value] = checked
	addon:UpdateAuraMode()
end