------------------------------------------------------------
-- PartyFrames.lua
--
-- Abin
-- 2012/1/03
------------------------------------------------------------

local _G = _G
local RegisterStateDriver = RegisterStateDriver
local UnregisterStateDriver = UnregisterStateDriver
local UnitExists = UnitExists

local _, addon = ...

local frame = CreateFrame("Frame", "CompactRaidPartyParentFrame", addon:GetMainFrame(), "SecureFrameTemplate")
frame:SetSize(16, 16)
frame:SetPoint("TOPLEFT")
RegisterStateDriver(frame, "visibility", "[group:raid] hide; [group:party] show; hide")
frame.buttons = {}

local petParent = CreateFrame("Frame", "CompactRaidPartyPetParentFrame", frame, "SecureFrameTemplate")
petParent.buttons = {}

local anchor
local i
for i = 0, 4 do
	local playerName = (i == 0 and "Player" or "Party"..i)
	local petName = (i == 0 and "Pet" or "PartyPet"..i)

	local playerButton = addon:CreateUnitFrame(frame:GetName()..playerName, frame, strlower(playerName))
	tinsert(frame.buttons, playerButton)
	if anchor then
		anchor:SetFrameRef("nextbutton", playerButton)
		playerButton:SetPoint("TOP", anchor, "TOP")
	else
		playerButton:SetPoint("TOPLEFT")
	end
	if i > 0 then
		playerButton.affectContainer = 1
	end

	local petButton = addon:CreateUnitFrame(frame:GetName()..petName, i == 0 and frame or petParent, strlower(petName), nil, true, "SecureHandlerShowHideTemplate")
	tinsert(petParent.buttons, petButton)
	petButton:SetAttribute("spacing", 1)
	petButton:Hide()
	petButton:SetPoint("TOP", playerButton, "BOTTOM", 0, -1)
	petButton.affectContainer = 1

	petButton:SetAttribute("_onshow", [[
		local nextbutton = self:GetFrameRef("nextbutton")
		if nextbutton then
			nextbutton:ClearAllPoints()
			nextbutton:SetPoint("TOP", self, "BOTTOM", 0, 0 - self:GetAttribute("spacing"))
		end
	]])

	petButton:SetAttribute("_onhide", [[
		local nextbutton = self:GetFrameRef("nextbutton")
		if nextbutton then
			nextbutton:ClearAllPoints()
			nextbutton:SetPoint("TOP", self, "TOP")
		end
	]])

	anchor = petButton
end

function addon:UpdatePartyMode()
	if addon.db.hideparty then
		UnregisterStateDriver(frame, "visibility")
		frame:Hide()

		local i
		for i = 1, 4 do
			local frame = _G["PartyMemberFrame"..i]
			RegisterStateDriver(frame, "visibility", "[nogroup] hide; [group:raid] hide; [@party"..i..",exists] show; hide")
			frame:SetAlpha(1)
			frame:SetScale(1)
		end
	else
		RegisterStateDriver(frame, "visibility", "[group:raid] hide; [group:party] show; hide")
		local i
		for i = 1, 4 do
			local frame = _G["PartyMemberFrame"..i]
			RegisterStateDriver(frame, "visibility", "hide")
			frame:SetAlpha(0)
			frame:SetScale(0.01)
		end
	end
end

function addon:UpdatePartyPetMode()
	if addon.db.hidepartypets then
		petParent:Hide()
	else
		petParent:Show()
	end
end

function addon:UpdatePartySpacing()
	local spacing = addon.db.spacing
	local i
	for i = 1, 5 do
		local playerButton = frame.buttons[i]
		local petButton = petParent.buttons[i]

		petButton:SetAttribute("spacing", spacing)
		petButton:ClearAllPoints()
		petButton:SetPoint("TOP", playerButton, "BOTTOM", 0, -spacing)

		if i > 1 then
			local prevPetButton = petParent.buttons[i - 1]
			playerButton:ClearAllPoints()
			if prevPetButton:IsVisible() then
				playerButton:SetPoint("TOP", prevPetButton, "BOTTOM", 0, -spacing)
			else
				playerButton:SetPoint("TOP", prevPetButton, "TOP")
			end
		end

	end
	addon:UpdateContainerSize()
end

function addon:GetPartyFrameVisibleSize()
	local count = 1
	if petParent.buttons[1]:IsVisible() then
		count = count + 0.5
	end

	local i
	for i = 2, 5 do
		if frame.buttons[i]:IsVisible() then
			count = count + 1
			if petParent.buttons[i]:IsVisible() then
				count = count + 0.5
			end
		else
			break
		end
	end

	return addon.db.width, addon.db.height * count + (count - 1) * addon.db.spacing
end