------------------------------------------------------------
-- ToolFrames.lua
--
-- Abin
-- 2012/1/03
------------------------------------------------------------

local _G = _G
local GetNumRaidMembers = GetNumRaidMembers
local IsRaidLeader = IsRaidLeader
local IsRaidOfficer = IsRaidOfficer
local IsPartyLeader = IsPartyLeader
local InitiateRolePoll = InitiateRolePoll
local DoReadyCheck = DoReadyCheck
local UnitExists = UnitExists
local GetRaidTargetIndex = GetRaidTargetIndex
local UnitPopupButtons = UnitPopupButtons
local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
local ToggleDropDownMenu = ToggleDropDownMenu
local UIErrorsFrame = UIErrorsFrame
local ERR_ARENA_TEAM_PERMISSIONS = ERR_ARENA_TEAM_PERMISSIONS

local _, addon = ...
local L = addon.L

local toolFrames = {}
local secureMenus = {}

local function CreateToolbox(name, r, g, b)
	local frame = CreateFrame("Button", name, addon:GetMainFrame(), "SecureHandlerClickTemplate,UIMenuButtonStretchTemplate")
	frame:SetSize(24, 15)
	tinsert(toolFrames, frame)

	local texture = frame:CreateTexture(frame:GetName().."Texture", "ARTWORK")
	texture:SetSize(10, 10)
	texture:SetPoint("CENTER", 0, -1)
	texture:SetTexture("Interface\\RaidFrame\\Raid-WorldPing")
	texture:SetVertexColor(r, g, b)

	frame:SetAttribute("_onclick", [[
		local menuframe = self:GetFrameRef("menuframe")
		local menucount = self:GetAttribute("securemenucount") or 0
		local i
		for i = 1, menucount do
			local securemenu = self:GetFrameRef("securemenu"..i)
			if securemenu and securemenu ~= menuframe then
				securemenu:Hide()
			end
		end

		if menuframe then
			if self:GetAttribute("menushown") then
				menuframe:Hide()
			else
				menuframe:Show()
			end
		end
	]])

	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", function(self) addon:StartMoving() end)
	frame:SetScript("OnDragStop", function(self) addon:StopMoving() end)

	return frame
end

local general = CreateToolbox("CompactRaidToolboxGeneral", 1, 1, 0)
addon.toolGeneral = general
general:SetPoint("BOTTOMLEFT", addon:GetMainFrame(), "TOPLEFT", -12, 11)

local worldMarker = CreateToolbox("CompactRaidToolboxWorldMarker", 0, 1, 0)
addon.toolWorldMarkr = worldMarker
worldMarker:SetPoint("LEFT", general, "RIGHT", -2, 0)
RegisterStateDriver(worldMarker, "visibility", "[group] show; hide")

local groupFilter = CreateToolbox("CompactRaidToolboxGroupFilter", 1, 1, 1)
addon.toolGroupFilter = groupFilter
groupFilter:SetPoint("LEFT", worldMarker, "RIGHT", -2, 0)
RegisterStateDriver(groupFilter, "visibility", "[group:raid] show; hide")

function addon:UpdateToolFramePosition()
	local size = addon.db.containerBorderSize
	general:ClearAllPoints()
	general:SetPoint("BOTTOMLEFT", addon:GetMainFrame(), "TOPLEFT", -size, size - 1)
end

function addon:UpdateToolFrameVisibility()
	if addon.db.classfilter == 1 then
		UnregisterStateDriver(groupFilter, "visibility")
		groupFilter:Hide()
	else
		RegisterStateDriver(groupFilter, "visibility", "[group:raid] show; hide")
	end
end

local menu = CreateFrame("Frame", general:GetName().."Menu", general, "UIDropDownMenuTemplate")
general.menu = menu
menu.point = "BOTTOMLEFT"
menu.relativeTo = general
menu.relativePoint = "BOTTOMRIGHT"
menu.xOffset = 0
menu.yOffset = 0

local function CheckPermission()
	local result = 1
	if GetNumRaidMembers() > 0 then
		result = IsRaidLeader() or IsRaidOfficer()
	end
	return result
end

local function TargetMarks_OnClick(self, arg1)
	if CheckPermission() then
		SetRaidTargetIcon(UnitExists("target") and "target" or "player", arg1 or 0)
	else
		UIErrorsFrame:AddMessage(ERR_ARENA_TEAM_PERMISSIONS, 1, 0, 0)
	end
end

local function DoGroupFunc(self, arg1)
	if arg1 then
		DoReadyCheck()
	else
		InitiateRolePoll()
	end
end

UIDropDownMenu_Initialize(menu, function(self, level, value)
	if level == 1 then
		local permission
		if GetNumRaidMembers() > 0 then
			permission = IsRaidLeader() or IsRaidOfficer()
		elseif GetNumPartyMembers() > 0 then
			permission = IsPartyLeader()
		end

		UIDropDownMenu_AddButton({ text = GENERAL, notCheckable = 1, isTitle = 1 })
		UIDropDownMenu_AddButton({ text = BINDING_HEADER_RAID_TARGET, notCheckable = 1, hasArrow = 1, menuList = "targetmarks" })
		UIDropDownMenu_AddButton({ text = ROLE_POLL, notCheckable = 1, disabled = not permission, func = DoGroupFunc })
		UIDropDownMenu_AddButton({ text = READY_CHECK, notCheckable = 1, disabled = not permission, func = DoGroupFunc, arg1 = 1 })
		UIDropDownMenu_AddButton({ text = SETTINGS.."...", notCheckable = 1, func = function() addon.optionFrame:Toggle() end })
		UIDropDownMenu_AddButton({ text = CLOSE, notCheckable = 1 })
	elseif level == 2 then
		if value == "targetmarks" then
			local unit = UnitExists("target") and "target" or "player"
			local mark = unit and GetRaidTargetIndex(unit)
			local i
			for i = 1, 8 do
				local data = addon:CloneTable(UnitPopupButtons["RAID_TARGET_"..i])
				data.checked = mark == i
				data.func = TargetMarks_OnClick
				data.arg1 = i
				UIDropDownMenu_AddButton(data, 2)
			end

			local data = addon:CloneTable(UnitPopupButtons["RAID_TARGET_NONE"])
			data.checked = not mark
			data.func = TargetMarks_OnClick
			UIDropDownMenu_AddButton(data, 2)
		end
	end
end, "MENU")

general:HookScript("OnClick", function(self)
	ToggleDropDownMenu(nil, nil, menu)
end)

local function Button_OnEnter(self) self.highlight:Show() end
local function Button_OnLeave(self) self.highlight:Hide() end

local function CreateSecureMenuButton(parent, text, templates, disabled)
	local button = CreateFrame("Button", nil, parent, templates)
	button:SetSize(parent:GetWidth() - 24, 18)

	local prev = parent.buttons[#parent.buttons]
	if prev then
		button:SetPoint("TOPLEFT", prev, "BOTTOMLEFT")
	else
		button:SetPoint("TOPLEFT", 12, -12 - 18)
	end

	tinsert(parent.buttons, button)

	local highlight = button:CreateTexture(nil, "BORDER")
	button.highlight = highlight
	highlight:Hide()
	highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	highlight:SetBlendMode("ADD")
	highlight:SetVertexColor(1, 1, 1, 0.7)
	highlight:SetAllPoints(button)
	button:HookScript("OnEnter", Button_OnEnter)
	button:HookScript("OnLeave", Button_OnLeave)

	local check = button:CreateTexture(nil, "ARTWORK")
	button.check = check
	check:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
	check:SetWidth(24)
	check:SetHeight(24)
	check:SetPoint("CENTER", button, "LEFT", 10, 0)
	check:Hide()

	local fs = button:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	button.text = fs
	fs:SetPoint("LEFT", check, "RIGHT")
	fs:SetText(text)

	if disabled then
		button:Disable()
		fs:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
		check:SetDesaturated(true)
	end

	return button
end

local function FinishSecureMenu(menu)
	local button = CreateSecureMenuButton(menu, CLOSE, "SecureHandlerClickTemplate")
	button:SetAttribute("_onclick", [[ self:GetParent():Hide() ]])
	menu:SetHeight((#menu.buttons + 1) * 18 + 24)
end

local function CreateSecureMenu(parent, width, title)
	local menu = CreateFrame("Button", nil, parent, "SecureHandlerShowHideTemplate,SecureHandlerClickTemplate")
	menu:Hide()
	menu.buttons = {}
	menu:SetPoint("BOTTOMLEFT", parent, "BOTTOMRIGHT")
	menu:SetWidth(width)
	menu:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }, })
	menu:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
	menu:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
	menu:SetToplevel(true)
	menu:SetFrameStrata("HIGH")
	menu:SetClampedToScreen(true)
	menu:RegisterForClicks("AnyUp")
	menu:SetAttribute("_onclick", [[ self:Hide() ]])
	menu:SetAttribute("_onshow", [[ self:GetParent():SetAttribute("menushown", 1) ]])
	menu:SetAttribute("_onhide", [[ self:GetParent():SetAttribute("menushown", nil) self:Hide() ]])
	parent:SetFrameRef("menuframe", menu)

	local titleText = menu:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	menu.title = titleText
	titleText:SetPoint("TOPLEFT", 34, -12)
	titleText:SetText(title)

	tinsert(secureMenus, menu)
	local count = #secureMenus

	local toolbox
	for _, toolbox in ipairs(toolFrames) do
		toolbox:SetAttribute("securemenucount", count)
		toolbox:SetFrameRef("securemenu"..count, menu)
	end

	return menu
end

local worldMarkerMenu = CreateSecureMenu(worldMarker, 132, L["world marks"])

local function WorldMarkButton_OnClick(self)
	if not CheckPermission() then
		UIErrorsFrame:AddMessage(ERR_ARENA_TEAM_PERMISSIONS, 1, 0, 0)
	end
end

local function WorldMarkerButton_OnUpdate(self, elapsed)
	self.elapsed = (self.elapsed or 0) + elapsed
	if self.elapsed > 0.2 then
		self.elapsed = 0
		if IsRaidMarkerActive(self.id) then
			self.check:Show()
		else
			self.check:Hide()
		end
	end
end

do
	local i
	for i = 1, 5 do
		local _, _, text = strfind(_G["WORLD_MARKER"..i], "(.+)|r(.+)")
		local button = CreateSecureMenuButton(worldMarkerMenu, text.."|r", "SecureActionButtonTemplate")
		button.id = i
		button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		button:SetAttribute("type", "macro")
		button:SetAttribute("macrotext1", "/wm "..i)
		button:SetAttribute("macrotext2", "/cwm "..i)
		button:SetScript("OnUpdate", WorldMarkerButton_OnUpdate)
		button:HookScript("OnClick", WorldMarkButton_OnClick)
	end

	local button = CreateSecureMenuButton(worldMarkerMenu, REMOVE_WORLD_MARKERS, "SecureActionButtonTemplate")
	button:SetAttribute("type", "macro")
	button:SetAttribute("macrotext", "/cwm "..ALL)
	button:HookScript("OnClick", WorldMarkButton_OnClick)

	FinishSecureMenu(worldMarkerMenu)
end

local groupFilterMenu = CreateSecureMenu(groupFilter, 120, RAID)

local function GroupFilterButton_UpdateCheck(self)
	if self.filterGroup:IsShown() then
		self.check:Show()
	else
		self.check:Hide()
	end
end

do
	local i
	for i = 1, 8 do
		local button = CreateSecureMenuButton(groupFilterMenu, format(GROUP_NUMBER, i), "SecureHandlerClickTemplate")
		button.check:Show()

		if i == 1 then
			button:Disable()
			button.text:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
			button.check:SetDesaturated(true)
		else
			local filterGroup = addon:GetRaidGroup(i)
			button.filterGroup = filterGroup
			button:SetFrameRef("filtergroup", filterGroup)
			button.UpdateCheck = GroupFilterButton_UpdateCheck
			button:SetAttribute("_onclick", [[
				local group = self:GetFrameRef("filtergroup")
				if group:IsShown() then
					group:Hide()
				else
					group:Show()
				end
				self:CallMethod("UpdateCheck")
			]])
		end
	end

	FinishSecureMenu(groupFilterMenu)
end